@@ -3,10 +3,13 @@ test_that("OptimizerAsyncRandomSearch works", {
   skip_if_not_installed("rush")
   flush_redis()
 
+  options(bbotk.tiny_logging = TRUE)
+
   optimizer = opt("async_random_search")
   expect_class(optimizer, "OptimizerAsync")
 
-  rush::rush_plan(n_workers = 2)
+  mirai::daemons(2)
+  rush::rush_plan(n_workers = 2, worker_type = "remote")
   instance = oi_async(
     objective = OBJ_2D,
     search_space = PS_2D,
